<?php

namespace Drupal\zammad_webform_handler\Plugin\WebformHandler;

use Drupal\Core\Url;
use ZammadAPIClient\Client;
use ZammadAPIClient\ResourceType;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform submission remote post handler.
 *
 * @WebformHandler(
 *   id = "zammad",
 *   label = @Translation("Zammad Ticket"),
 *   category = @Translation("External"),
 *   description = @Translation("Posts webform submissions to Zammad as new Ticket."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_OPTIONAL,
 *   tokens = TRUE,
 * )
 */
class ZammadWebformHandler extends WebformHandlerBase {

  /**
   * The token manager.
   *
   * @var \Drupal\webform\WebformTokenManagerInterface
   */
  protected $tokenManager;

  /**
   * The zammad_webform_handler config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $externalConfig;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->externalConfig = $container->get('config.factory')->get('zammad_webform_handler.settings');
    $instance->tokenManager = $container->get('webform.token_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'ticket_title' => '',
      'ticket_body' => '[webform_submission:values]',
      'ticket_group' => '',
      'ticket_customer_id' => '',
      'ticket_customer_mail' => '',
      'ticket_priority_id' => '',
      'ticket_state_id' => '',
      'ticket_type' => '',
      'ticket_subject' => '',
      'ticket_internal' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['zammad_webform_handler_settings'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Zammad Handler Settings'),
      '#description' => $this->t('Make sure the authentication method, the zammad url and the http token are set and make sure your zammad installation supports Zammad API Calls!'),
    ];
    $form['zammad_webform_handler_settings']['link'] = [
      '#type' => 'link',
      '#title' => $this->t('General Zammad settings page (API-Key / URLs)'),
      '#url' => Url::fromRoute('zammad_webform_handler.settings_form'),
      '#attributes' => [
        'class' => ['button'],
        'target' => '_blank',
      ],
    ];
    $form['zammad_webform_handler_settings']['required_settings'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Required Settings'),
    ];
    $form['zammad_webform_handler_settings']['optional_settings'] = [
      '#type' => 'details',
      '#open' => FALSE,
      '#title' => $this->t('Optional Settings'),
    ];
    $form['zammad_webform_handler_settings']['api_link'] = [
      '#type' => 'link',
      '#title' => $this->t('Zammad API: Ticket Documentation'),
      '#url' => Url::fromUri('https://docs.zammad.org/en/latest/api/ticket/index.html'),
    ];
    // REQUIRED SETTINGS:
    $form['zammad_webform_handler_settings']['required_settings']['ticket_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Title'),
      '#default_value' => $this->configuration['ticket_title'],
      '#description' => $this->t('The Zammad Ticket Title text, e.g. "[webform_submission:values:subject] ([webform:title] - [site:url])"') . ' ' . $this->t('This field supports tokens.'),
      '#required' => TRUE,
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_body'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Zammad Ticket Body'),
      '#default_value' => $this->configuration['ticket_body'],
      '#description' => $this->t('The Zammad Ticket Body text, e.g. [webform_submission:values:message]') . ' ' . $this->t('This field supports tokens.'),
      '#required' => TRUE,
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_group'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Group'),
      '#default_value' => $this->configuration['ticket_group'],
      '#description' => $this->t('The Zammad Group, e.g. "Sales"') . ' ' . $this->t('This field supports tokens.'),
      '#required' => TRUE,
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_customer_mail'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Create ticket on behalf of customer eMail (or find / create automatically)'),
      '#description' => $this->t('If the eMail address does not exist, a user with the mail in question gets created.') . ' ' . $this->t('This field supports tokens.'),
      '#default_value' => $this->configuration['ticket_customer_mail'],
      '#states' => [
        'visible' => [
          ':input[name="settings[ticket_customer_id]"]' => ['filled' => FALSE],
        ],
      ],
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_customer_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Create ticket on behalf of Customer ID (leave empty for eMail detection)'),
      '#description' => $this->t('Leave empty to use the eMail address auto-detection/creation feature!') . ' ' . $this->t('This field supports tokens.'),
      '#default_value' => $this->configuration['ticket_customer_id'],
      '#states' => [
        'visible' => [
          ':input[name="settings[ticket_customer_mail]"]' => ['filled' => FALSE],
        ],
      ],
    ];

    // OPTIONAL SETTINGS:
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Subject'),
      '#default_value' => $this->configuration['ticket_subject'],
      '#description' => $this->t('This field supports tokens.'),
    ];
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_priority_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Priority ID'),
      '#description' => $this->t('This field supports tokens.'),
      '#default_value' => $this->configuration['ticket_priority_id'],
    ];
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_state_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket State ID'),
      '#description' => $this->t('This field supports tokens.'),
      '#default_value' => $this->configuration['ticket_state_id'],
    ];
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_type'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Type'),
      '#description' => $this->t('This field supports tokens.'),
      '#default_value' => $this->configuration['ticket_type'],
    ];
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_internal'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Whether or not it is an internal ticket.'),
      '#default_value' => $this->configuration['ticket_internal'],
    ];

    $this->elementTokenValidate($form);

    return $this->setSettingsParents($form);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    if (empty($this->externalConfig->get('authentication_method'))) {
      $form_state->setErrorByName('zammad_webform_handler_settings_link', $this->t('Please choose a authentication method on the settings page!'));
    }
    if (empty($this->externalConfig->get('zammad_url'))) {
      $form_state->setErrorByName('zammad_webform_handler_settings_link', $this->t('Please define a zammad url on the settings page!'));
    }
    if (empty($this->externalConfig->get('http_token_secret'))) {
      $form_state->setErrorByName('zammad_webform_handler_settings_link', $this->t('Please define a HTTP Token on the settings page!'));
    }
    if (empty($form_state->getValue('ticket_customer_id')) && empty($form_state->getValue('ticket_customer_mail'))) {
      $form_state->setErrorByName('ticket_customer_mail', $this->t('Either the Zammad Ticket Customer Mail or the Zammad Ticket Customer ID should be set!'));
    }
    return parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE): void {
    $this->sendZammadTicket($webform_submission);
  }

  /**
   * Sends a Zammad Ticket a zammad installation.
   */
  protected function sendZammadTicket(WebformSubmissionInterface $webform_submission): void {
    if (empty($this->externalConfig->get('authentication_method')) || empty($this->externalConfig->get('zammad_url')) || empty($this->externalConfig->get('http_token_secret'))) {
      $this->getLogger('zammad_webform_handler')
        ->error('Error: The authentication_method, zammad_url or http_token_secret are empty.');
      return;
    }

    $timeout = $this->externalConfig->get('timeout');
    if (empty($timeout)) {
      $timeout = 5;
    }

    $client = new Client([
      'url' => $this->externalConfig->get('zammad_url'),
      'http_token' => $this->externalConfig->get('http_token_secret'),
      'timeout' => $timeout,
      'verify' => $this->externalConfig->get('verify'),
    ]);

    $ticket = $client->resource(ResourceType::TICKET);

    // Tickets are always created on behalf of a user / customer.
    // Use "guess:"-functionality, if customer_id is empty:
    // See https://docs.zammad.org/en/latest/api/ticket/index.html:
    // If you want to create tickets on behalf other users,
    // use the customer_id attribute.
    // Use guess:{email address} to save an API call if you don’t know the users
    // ID or want to create the user in question.
    $customerId = $this->replaceTokens($this->configuration['ticket_customer_id'], $webform_submission);
    $customerMail = $this->replaceTokens($this->configuration['ticket_customer_mail'], $webform_submission);
    if (empty($customerId)) {
      if (!empty($customerMail)) {
        $customerId = "guess:" . $customerMail;
      }
    }

    $group = $this->replaceTokens($this->configuration['ticket_group'], $webform_submission);
    $title = $this->replaceTokens($this->configuration['ticket_title'], $webform_submission);
    $body = $this->replaceTokens($this->configuration['ticket_body'], $webform_submission);

    // Setup required ticket data:
    $ticket_data = [
      'title' => $title,
      'group' => $group,
      'customer_id' => $customerId,
      'article' => [
        'body' => $body,
      ],
    ];

    // Add optional ticket data:
    if (!empty($type = $this->configuration['ticket_type'])) {
      if (!empty($replacedType = $this->replaceTokens($type, $webform_submission))) {
        $ticket_data['article']['type'] = $replacedType;
      }
    }
    if (!empty($priorityId = $this->configuration['ticket_priority_id'])) {
      if (!empty($replacedPriorityId = $this->replaceTokens($priorityId, $webform_submission))) {
        $ticket_data['ticket_priority_id'] = $replacedPriorityId;
      }
    }
    if (!empty($stateId = $this->configuration['ticket_state_id'])) {
      if (!empty($replacedStateId = $this->replaceTokens($stateId, $webform_submission))) {
        $ticket_data['state_id'] = $replacedStateId;
      }
    }
    if (!empty($subject = $this->configuration['ticket_subject'])) {
      if (!empty($replacedSubject = $this->replaceTokens($subject, $webform_submission))) {
        $ticket_data['article']['subject'] = $replacedSubject;
      }
    }
    if ($internal = $this->configuration['ticket_internal']) {
      $ticket_data['article']['ticket_internal'] = $internal;
    }
    // Set ticket values and send ticket:
    $ticket->setValues($ticket_data);
    $ticket->save();
    if ($ticket->hasError()) {
      $this->getLogger('zammad_webform_handler')
        ->error($ticket->getError());
    }
  }

}
